"""
    This file is part of immanuel - (C) The Rift Lab
    Author: Robert Davies (robert@theriftlab.com)


    Defines indices for the chart classes to define what data should be
    included for output. These are string rather than integer indices due to
    the way the chart classes use these to name their functions.

"""


NATIVE = 'native'
PARTNER = 'partner'
SOLAR_RETURN_YEAR = 'solar_return_year'
SOLAR_RETURN_DATE_TIME = 'solar_return_date_time'
PROGRESSION_DATE_TIME = 'progression_date_time'
PROGRESSED_DATE_TIME = 'progressed_date_time'
PROGRESSION_METHOD = 'progression_method'
HOUSE_SYSTEM = 'house_system'
SHAPE = 'shape'
DIURNAL = 'diurnal'
MOON_PHASE = 'moon_phase'
OBJECTS = 'objects'
ASPECTS = 'aspects'
HOUSES = 'houses'
WEIGHTINGS = 'weightings'
